#include "fw/dynmx/protocol.h"
#include "fw/serial.h"

#include <schpin_lib_exp/protocol.h>

namespace schpin
{

struct dynmx_serial
{
        serial_connection                                     ser;
        std::array< uint8_t, dynmx_master_message::max_size > buffer;

        dynmx_serial( serial_connection conn )
          : ser( conn )
        {
        }

        void send( const master_dynmx_message& m )
        {
                ser.write( m );
        }

        void log_error()
        {
                SCHPIN_ERROR_LOG(
                    "s", "Error " << errno << " when console: " << strerror( errno ) );
        }

        dynmx_master_message read_reply()
        {
                std::size_t wanted = 7;
                std::size_t got    = 0;

                while ( got != wanted ) {
                        ssize_t n = ser.read( buffer.begin() + got, wanted - got );
                        if ( n < 0 ) {
                                log_error();
                                continue;
                        }
                        got += static_cast< std::size_t >( n );
                        if ( got < 7 ) {
                                continue;
                        }
                        protocol_item< dynmx_packet_size >::deserialize(
                            em::view_n( buffer.begin() + 5, 2 ),
                            match(
                                [&]( error_tag, auto ) {
                                        SCHPIN_ERROR_LOG(
                                            "dynmx.protocol", "Parsed packet length is invalid" );
                                },
                                [&]( value_tag, dynmx_packet_size val ) {
                                        wanted += *val;
                                } ) );
                }

                return dynmx_master_message{ buffer, got };
        }
};

}  // namespace schpin
