#include "fw/util.h"

#pragma once

namespace schpin
{
class lewan_converter
{
        float in_from = 0.f;
        float in_to   = 0.f;
        float se_from = 0.f;
        float se_to   = 0.f;

public:
        lewan_converter() = default;

        lewan_converter( bangle i_from, bangle i_to, lewan_angle s_from, lewan_angle s_to )
          : in_from( i_from.as_float() )
          , in_to( i_to.as_float() )
          , se_from( static_cast< float >( *s_from ) )
          , se_to( static_cast< float >( *s_to ) )
        {
        }

        std::optional< lewan_angle > in_to_se( bangle a ) const
        {
                float mapped = map_val< float >( a.as_float(), in_from, in_to, se_from, se_to );
                return lewan_angle::make( mapped );
        }

        std::optional< lewan_time > in_to_se( btime t ) const
        {
                return lewan_time::make( t.as_float() * 1000.f );
        }

        std::optional< lewan_angular_velocity > in_to_se( bangular_velocity av ) const
        {
                float s_diff = se_to - se_from;
                float i_diff = in_to - in_from;
                float coeff  = std::fabs( s_diff ) / std::fabs( i_diff );

                float res = av.as_float() * coeff;

                if ( em::sign( s_diff ) != em::sign( i_diff ) ) {
                        res *= -1;
                }

                return lewan_angular_velocity::make( res );
        }

        bangle se_to_in( lewan_full_angle a ) const
        {
                float mapped =
                    map_val< float >( static_cast< float >( *a ), se_from, se_to, in_from, in_to );
                return bangle::from_float( mapped );
        }

        bangle se_to_in( lewan_angle a ) const
        {
                return se_to_in( lewan_full_angle{ a } );
        }

        btemp se_to_in( lewan_temp temp ) const
        {
                return btemp::from_float( *temp );
        }

        bvoltage se_to_in( lewan_voltage volt ) const
        {
                return bvoltage{ *volt };
        }

        bangular_velocity se_to_in( lewan_angular_velocity vel ) const
        {
                float mapped = map_val< float >(
                    static_cast< float >( *vel ), se_from, se_to, in_from, in_to );

                return bangular_velocity::from_float( mapped );
        }
};
}  // namespace schpin
