#include "shared/protocol.h"

#pragma once

namespace schpin
{

template < typename T, typename A >
inline auto handle_in_to_se( const T& conv, ctl_id cid, messages_enum ID, A a_val )
    -> em::either< std::decay_t< decltype( *conv.in_to_se( a_val ) ) >, servo_master_message >
{
        auto opt_a = conv.in_to_se( a_val );
        if ( !opt_a ) {
                return make_servo_error_message< INPUT_INVALID_VALUE_E >( cid, ID, 0u );
        }

        return *opt_a;
}

template < typename T, typename A, typename B >
inline auto handle_in_to_se( const T& conv, ctl_id cid, messages_enum ID, A a_val, B b_val )
    -> em::either<
        std::tuple< decltype( *conv.in_to_se( a_val ) ), decltype( *conv.in_to_se( b_val ) ) >,
        servo_master_message >
{

        auto opt_a = conv.in_to_se( a_val );
        if ( !opt_a ) {
                return make_servo_error_message< INPUT_INVALID_VALUE_E >( cid, ID, 0u );
        }

        auto opt_b = conv.in_to_se( b_val );
        if ( !opt_b ) {
                return make_servo_error_message< INPUT_INVALID_VALUE_E >( cid, ID, 1u );
        }

        return std::make_tuple( *opt_a, *opt_b );
}

}  // namespace schpin
