#include "node/load.h"

#include <schpin_lib_exp/config.h>

using namespace schpin;

em::either< std::tuple< node_hardware_config, robot_model >, load_error >
schpin::load_node_data( rclcpp::Node& node )
{
        return get_config< node_hardware_config >( node, "" )
            .convert_right( [&]( const config_error& e ) {
                    return E( load_error ).attach( e )
                        << "Failed to load app due to erros in config parse";
            } )
            .bind_left( [&]( node_hardware_config conf ) {
                    urdf::Model model;
                    model.initString( conf.robot_description );

                    return load_srdf_info( conf.robot_semantics )
                        .convert_right( [&]( const xml_storage_error& e ) {
                                return E( load_error ).attach( e )
                                    << "Failed to load due to erros with srdf";
                        } )
                        .bind_left( [&]( const srdf_information& srdf_info ) {
                                return load_robot_model( model, srdf_info )
                                    .convert_left( [&]( robot_model r ) {
                                            return std::make_tuple( conf, std::move( r ) );
                                    } );
                        } );
            } );
}
